#!/bin/perl
################################################################################
# Created       : Martin Foster
# Modified      : 28/11/2002
################################################################################
#
# Registration - Script part of Webchat designed to register patrons 
# Copyright (C) 2000-2002  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : martin@ethereal-realms.org
#       Address : 204, 817 5TH ST NE
#                 Calgary, Alberta
#                 T2E 3W9
#
################################################################################
use CGI qw(:standard);						# Common gateway interface
use CGI::Carp qw(fatalsToBrowser);				# CGI Error logs
use Digest::MD5 qw(md5_hex);					# MD5 digest cryptographic algorithm
use strict;							# Strict variable enforcement

use Ethereal::Database;						# Database handler
use Ethereal::Login;						# Login functionality
use Ethereal::Mail;						# Mail handling
use Ethereal::Option;						# Option handling
use Ethereal::Param;						# Parameter control
use Ethereal::Table;						# Table module
use Ethereal::Template;						# Template handler


#################################################################################
# Data Members
#################################################################################
my $cgi;							# Common gateway interface handle
my $database;							# Database handle
my $login;							# Login handle
my $send;							# Mail name
my $param;							# Parameter hash

my $capture;							# Arbitrary captured line
my $hostname;							# Remote host hostname
my $scriptname;							# Name of called script

my %login;							# Login hash
my %system;							# System hash


#################################################################################
# Program Area
#################################################################################

	
	# Initial handles
	$cgi      = new CGI;
	$database = new Ethereal::Database();

	# Establish connection
	$database->Connect();

	# Parameter handling
	$param = new Ethereal::Param($database, $cgi);


	# Retreive hostname
	$hostname = $cgi->remote_host();


	# Retreive hash
	$database->GetHashLogin(\%login);


	# Entrance without parameters
	unless ($cgi->param())
	{
		# Pull scriptname
		$scriptname = $cgi->url(-relative=>1);


		# Headers
		print $cgi->header();
		$database->DocumentGetHeader();

		# Display initial form
		if ($scriptname =~ /^confirmation/i)
		{
			# Confirmation form
			InformationConf($database, $cgi, $param);
		}
		else
		{
			# Registration form
			InformationFormInit($database, $cgi, $param);
		}


		# Footer
		$database->DocumentGetFooter();
	}

	# Entrace with parameters
	else
	{
		# Retreive parameters
		$param->GetParam();

		# If no registration codes or logins		
		unless ((defined($param->{'LOGIN'})) && (defined($param->{'CONFCODE'})))
		{
			# Headers
			print $cgi->header();
			$database->DocumentGetHeader();

			# If not accepted show AUP
			unless (defined($param->{'ACCEPT'}))
			{ 
				# Check contrains if correct then print AUP
				if (ConstraintHandler($database, $cgi, $param))
				{ PolicyForm($database, $cgi, $param); }
			}

			# If accepted process request
			elsif (defined($param->{'ACCEPT'}))
			{
				RequestHandler($database, $cgi, $param);
			}

			# Footer
			$database->DocumentGetFooter();
		}

		# If such a code was sent
		else
		{ 
			# Check if registration is valid
			if (AcceptHandler($database, $param))
			{
				# HTML Display
				# Headers
				print $cgi->header();
				$database->DocumentGetHeader();

				# Display fact they were accepted
				$database->DocumentGetAccepted(); 

				# Footer
				$database->DocumentGetFooter();


				# Email notification
				# Retreive Hash
				$database->GetHashSystem(\%system);

				# Retreive information
				my ($verified) = $database->DocumentGetConfirmed();
				my ($email)    = $database->DataGetPuppeteerEmail($param->{'LOGIN'});
				my ($name)     = $database->DataGetPuppeteerName($param->{'LOGIN'});


				# Mail sending
				# Initial handle
				$send = tie(*MAIL, 'Ethereal::Mail', $database);
				
				# Setup
				$send->SetSubject("Welcome to $system{SetInfoServer}");
				$send->SetSearch(
					EMAIL      => $email,
					FULLNAME   => $name,
					HOSTNAME   => $hostname,
					LOGINNAME  => $param->{'LOGIN'},
					PORTAL     => $system{'LocScriptPortal'},
					SERVERNAME => $system{'SetInfoServer'}
				 );

				# Recipients
				$send->AddTo("$name <$email>");
				$send->AddBCC("$system{SetSubmissionName} <$system{SetSubmissionAddress}>");

				# Message
				print MAIL $verified;

				# Close/Send
				close(MAIL);
			}
			
			# If not redirect to portal
			else
			{ print $cgi->redirect($login{'LnkPortal'}); }

		}
	}


################################################################################
# Sub-Routines
################################################################################

#####################
# Acceptence Handler
#
# Will determine is registration code is correct.  If so, the script will remove
# registration information allowing user to login as needed.

sub AcceptHandler
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $param    = shift;					# Parameter list

	my $choice;						# Captured data
	my $data     = $database->{'HANDLE'};			# Handle name

	#####################
	# Program area

	# Retreive login
	($choice) = $database->DataGet("SELECT PuppeteerLogin
		FROM  PuppeteerRegistration
		WHERE PuppeteerLogin=?
		  AND PuppeteerConfCode=?",
	 $param->{'LOGIN'},
	 $param->{'CONFCODE'});
	
	# If entry was found
	if (defined($choice))
	{
		# Erase lock
		$data->do("DELETE 
			FROM PuppeteerRegistration 
			WHERE PuppeteerLogin=? 
			  AND PuppeteerConfCode=?", {}, 

		 $param->{'LOGIN'}, 
		 $param->{'CONFCODE'});

		# Return true
		return 1;
	}

	return 0;
}


#####################
# Check Parameter
# 
# Checks parameters and verifies if data exists already in table or if they
# fit criteria.

sub CheckParam
{
	#####################
	# Data members
	my $subject = shift; 					# Arbitrary retrieval
	my $choice  = shift;					# Choice on the matter
	my $check   = shift;					# Check value

	#####################
	# Program area

	# Lenght
	if ($choice == 0)
	{
		if (length($subject) < 3)
		{ return 0; }					# False if password short

		return 1;					# Return true if 5 or more
	}

	# E-Mail
	if ($choice == 1)
	{
		# Basic verification
		if ($check !~ /^\w(\w|\.|\-)*?\@(\w|\-)*?\..*?/s)
		{ return 0; }

		# Retreive Email
		($choice) = $subject->DataGet("SELECT PuppeteerEmail
			FROM  Puppeteer
			WHERE PuppeteerEmail=?", $check);

		# Determin $choice and return
		$choice = (defined($choice)) ? 0 : 1;
		return $choice;
	}

	# Login name availability
	elsif ($choice == 2)
	{
		# Retreive Name
		($choice) = $subject->DataGetPuppeteerName($check);

		# Determin $choice and return
		$choice = (defined($choice)) ? 0 : 1;
		return $choice;
	}
}

# Alias for Lenght
sub CheckParamLenght
{ return CheckParam(shift, 0); }

# Alias for Email
sub CheckParamEmail
{ return CheckParam(shift, 1, shift); }

# Alias for Login
sub CheckParamLogin
{ return CheckParam(shift, 2, shift); }


#####################
# Constraint Handler
#
# Checks contraints, if they are correct returns true, otherwise requests
# repost of data missing or off.

sub ConstraintHandler						# Database, CGI, Parameter Reference
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter reference


	#####################
	# Program area

	# Username/login verfication
	unless ((CheckParamLogin($database, $param->{'USERNAME'})) 
	 && (CheckParamLenght($param->{'USERNAME'})))		
	{ InformationFormLogin($database, $cgi, $param); return 0;}	# Return false

	# Full name verification
	unless (CheckParamLenght($param->{'FULLNAME'}))
	{ InformationFormFullname($database, $cgi, $param); return 0;}	# Return false

	# EMail check
	unless (($param->{'EMAIL'} eq $param->{'EMAILTWICE'}) 
	 && (CheckParamLenght($param->{'EMAIL'})) 
	 && (CheckParamEmail($database, $param->{'EMAIL'})))
	{ InformationFormEmail($database, $cgi, $param);  return 0;}	# Return false

	# Password verification
	unless (($param->{'PASSONCE'} eq $param->{'PASSTWICE'}) 
	 && (CheckParamLenght($param->{'PASSONCE'})))
	{ InformationFormPassword($database, $cgi, $param); return 0;}	# Return false

	return 1;
}


#####################
# Information Conf
#
# Information form for confirmation purposes.   This is called when you reach
# a link with confirmation in the start of it's name.  Will essentially allow
# one to bypass bad Email links.

sub InformationConf
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter handling

	my $table    = new Ethereal::Table($cgi);		# Table handler
	my $tmpl;						# Template handler

	my %login;						# Login hash


	#####################
	# Program area

	# Link to hashes
	$database->GetHashLogin(\%login);

	# Template handler
	$tmpl = new Ethereal::Template(\%login);


	# Information on registration
	$database->DocumentGetConfirmation();

	# Start form
	print $cgi->start_form();

	# Top and widgets
	$table->MakeTop();
	$table->MakeValid(
	 $tmpl->PassTitle($login{'TagUsername'}),
	 $cgi->textfield('LOGIN', '', 10, 10));
	$table->MakeValid(
	 $tmpl->PassTitle($login{'TagConfirm'}),
	 $cgi->textfield('CONFCODE', '', 10, 10));
	$table->MakeBlank();

	# Button and bottom
	$table->MakeSingle($cgi->submit($login{'TxtConfirm'}));
	$table->MakeBottom();

	# End form
	print $cgi->end_form();
}

#####################
# Information Form
#
# Initially displayed sub-routine, for sole purpose of gathering infromation.

sub InformationForm
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter handling
	my $choice   = shift;					# Choice of the matter

	my $option   = new Ethereal::Option();			# Option handler
	my $table    = new Ethereal::Table($cgi);		# Table handler
	my $tmpl;						# Template handler

	my $javascript;						# Javascript option
	my $chatpref;						# Chatpref option
	my $formatting;						# Formatting option

	my @javascript;						# Javascript prefs array
	my @chatpref;						# Chat prefs array
	my @formatting;						# Formatting options

	my %login;						# Login hash
	my %system;						# System hash


	#####################
	# Program area

	# Link to hashes
	$database->GetHashLogin(\%login);
	$database->GetHashSystem(\%system);

	# Template handler
	$tmpl = new Ethereal::Template(\%login);


	# Retreive arrays with necessary options
	$option->Split(\@javascript, $login{'OptJavascript'});
	$option->Split(\@chatpref, $login{'OptChatPref'});
	$option->Split(\@formatting, $login{'OptFormat'});

	# Information on registration
	$database->DocumentGetRegistration();


	# Begin form
	print $cgi->start_form();

	# Main information
	$table->MakeTop();


	# Username
	# Initial
	if ($choice == 99)
	{ $table->MakeValid(
	   $tmpl->PassTitle($login{'TagUsername'}),
	   $cgi->textfield('USERNAME', '', 10, 10)); }

	# Invalid
	elsif ($choice == 0)
	{ $table->MakeInvalid(
	   $tmpl->PassTitle($login{'TagUsername'}),
	   $cgi->textfield('USERNAME', '', 10, 10)); }

	# Valid repetition
	else	
	{ $table->MakeValid(
	   $tmpl->PassTitle($login{'TagUsername'}),
	   $cgi->textfield('USERNAME', $param->{'USERNAME'}, 10, 10)); }


	# Fullname
	# Initial
	if ($choice == 99)
	{ $table->MakeValid(
	   $tmpl->PassTitle($login{'TagFullName'}),
           $cgi->textfield('FULLNAME', '', 20, 30)); }

	# Invalid
	elsif ($choice == 1)
	{ $table->MakeInalid(
	   $tmpl->PassTitle($login{'TagFullName'}),
           $cgi->textfield('FULLNAME', '', 20, 30)); }

	# Valid repetition
	else	
	{ $table->MakeValid(
	   $tmpl->PassTitle($login{'TagFullName'}),
           $cgi->textfield('FULLNAME', $param->{'FULLNAME'}, 20, 30)); }


	# Spacer
	$table->MakeBlank();

	# EMail
	# Initial
	if ($choice == 99)
	{ 
		$table->MakeValid(
		 $tmpl->PassTitle($login{'TagEMail'}),
		 $cgi->textfield('EMAIL', '', 20, 45));
	 	$table->MakeValid(
		 $tmpl->PassTitle($login{'TagEMailRepeat'}),
		 $cgi->textfield('EMAILTWICE', '', 20, 45));
	}

	# Invalid
	elsif ($choice == 2)
	{ 
		$table->MakeInvalid(
		 $tmpl->PassTitle($login{'TagEMail'}),
		 $cgi->textfield('EMAIL', '', 20, 45)); 
		$table->MakeInvalid(
		 $tmpl->PassTitle($login{'TagEMailRepeat'}),
		 $cgi->textfield('EMAILTWICE', '', 20, 45)); 
	}

	# Valid repetition
	else
	{ 
		$table->MakeValid(
		 $tmpl->PassTitle($login{'TagEMail'}),
		 $cgi->textfield('EMAIL', $param->{'EMAIL'}, 20, 45)); 
		$table->MakeValid(
		 $tmpl->PassTitle($login{'TagEMailRepeat'}),
		 $cgi->textfield('EMAILTWICE', $param->{'EMAIL'}, 20, 45)); 
	}


	# Spacer
	$table->MakeBlank();

	# Password information
	# Initial
	if ($choice == 99)
	{
		$table->MakeValid(
		 $tmpl->PassTitle($login{'TagPassword'}),
		 $cgi->password_field('PASSONCE', '', 10, 15));
		$table->MakeValid(
		 $tmpl->PassTitle($login{'TagPasswordRepeat'}),
		 $cgi->password_field('PASSTWICE', '', 10, 15));
	}

	# Invalid
	elsif ($choice == 3)
	{
		$table->MakeInvalid(
		 $tmpl->PassTitle($login{'TagPassword'}),
		 $cgi->password_field('PASSONCE', '', 10, 15));
		$table->MakeInvalid(
		 $tmpl->PassTitle($login{'TagPasswordRepeat'}),
		 $cgi->password_field('PASSTWICE', '', 10, 15));
	}

	# Valid Repeat
	else	
	{
		$table->MakeValid(
		 $tmpl->PassTitle($login{'TagPassword'}),
		 $cgi->password_field('PASSONCE', $param->{'PASSONCE'}, 10, 15));
		$table->MakeValid(
		 $tmpl->PassTitle($login{'TagPasswordRepeat'}),
		 $cgi->password_field('PASSTWICE', $param->{'PASSTWICE'}, 10, 15));
	}
		
	# Spacer
	$table->MakeBlank();
	$table->MakeBottom();

	# Initialize Javascript and Chat preferences
	$javascript = $cgi->radio_group('JAVASCRIPT', \@javascript, $javascript[0],'true');
	$chatpref   = $cgi->radio_group('CHATPREFS', \@chatpref, $chatpref[2],'true');
	$formatting = $cgi->radio_group('FORMATPREFS', \@formatting, $formatting[0],'true');

	# Columned
	$table->MakeTop();
	
	# Title and options
	$table->MakeTertiary(
	 $tmpl->PassSub($login{'TagOptJavacript'}),
	 $tmpl->PassSub($login{'TagOptChatPref'}),
	 $tmpl->PassSub($login{'TagOptFormat'}));
	$table->MakeTertiary($javascript, $chatpref, $formatting);
	$table->MakeBlank();
	

	# Submission
	$table->MakeValid($cgi->submit($login{'TxtSubmitButtonSub'}), $cgi->reset($login{'TxtResetButton'}));
	$table->MakeBottom();
	
	print $cgi->end_form(), "\n";				# End form
}

# Alias for Initilization
sub InformationFormInit
{ InformationForm(shift, shift, shift, 99); }

# Alias for Login
sub InformationFormLogin
{ InformationForm(shift, shift, shift, 0); }

# Alias for Fullname
sub InformationFormFullname
{ InformationForm(shift, shift, shift, 1); }

# Alias for EMail
sub InformationFormEmail
{ InformationForm(shift, shift, shift, 2); }

# Alias for pasword
sub InformationFormPassword
{ InformationForm(shift, shift, shift, 3); }

#####################
# Policy Form
#
# Displays the acceptable use policy flag and embeds all information gathered.
# Also adds a tag to mark acceptance.

sub PolicyForm							# Database, CGI, Parameter hash
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# Common gateway interface
	my $param    = shift;					# Parameter

	my $single;						# Single unit

	my %login;						# Login hash dbm


	#####################
	# Program area

	$database->GetHashLogin(\%login);			# Link with hash
	$database->DocumentGetAUP();				# Display Acceptable use Policy


	# Cleanup
	$param->Cleanup($login{'TxtSubmitButtonSub'});


	# Form display
	print $cgi->start_form();
	print $param->EmbedNormal($param->Flat(), 'ACCEPT', 'True'), "\n";

	# Print access button
	print "\t", $cgi->submit($login{'TxtSubmitButtonAccept'}), "\n";
	
	# End of form
	print $cgi->end_form(), "\n";
}


#####################
# Request Handler
#
# Takes in the parameters, sending a part of the information to the database, then
# sending a confirmation E-Mail to the poster.

sub RequestHandler						# Database, CGI, Parameter hash
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# Common Gateway Interface
	my $param    = shift;					# Parameter hash

	my $confcode;						# Confirmation code
	my $inline;						# Authenthication string
	my $invitation;						# Invitation	

	my $data     = $database->{'HANDLE'};			# Database connection handle
	my $option   = new Ethereal::Option();			# Options handler
	my $send;						# Mail sender

	my %system;						# System hash to DBM
	my %login;						# Login hash to DBM


	#####################
	# Program area

	# Link to hashes
	$database->GetHashLogin(\%login);
	$database->GetHashSystem(\%system);

	# Information on registration
	$database->DocumentGetRegistration();	
	$invitation = $database->DocumentGetInvitation();

	# Confirmation code
	($confcode) = $database->DataGet("SELECT PuppeteerConfCode
		FROM PuppeteerRegistration
		WHERE PuppeteerLogin=?", $param->{'USERNAME'});

	# Already exists
	unless (defined($confcode))
	{
		# Generate new code
		$confcode = md5_hex(rand());
		$confcode = substr($confcode, 0, 10);
	}


	# Create authenthication string
	$inline = $param->Crypt('LOGIN', $param->{'USERNAME'}, 'CONFCODE', $confcode);


	# Mail sending
	# Initiate handle
	$send = tie(*MAIL, 'Ethereal::Mail', $database);

	# Initial setup
	$send->SetSubject("An invitation to $system{SetInfoServer}");
	$send->SetSearch(
		AUTHSTRING  => "$system{LocScriptConf}$inline",
		CONFADDR    => $system{'LocScriptConf'},
		CONFCODE    => $confcode,
		FULLNAME    => $param->{'FULLNAME'},
		LOGINNAME   => $param->{'USERNAME'},
		PORTAL      => $system{'LocScriptPortal'},
		RETURNEMAIL => $system{'SetInfoContactAddress'},
		SERVERNAME  => $system{'SetInfoServer'}
	 );

	# Recipients
	$send->AddTo("$param->{'FULLNAME'} <$param->{EMAIL}>");

	# Message
	print MAIL $invitation;

	# Close and send 
	close(MAIL);
	

	# Puppeteers
	# Insert Puppeteer
	$data->do("INSERT INTO Puppeteer 
		(PuppeteerLogin,
		 PuppeteerPassword,
		 PuppeteerName,
		 PuppeteerEMail,
		 PuppeteerTimestamp,
		 PuppeteerFormatting,
		 PuppeteerChatPref,
		 PuppeteerJavascript,
		 PuppeteerAdminPrivs,
		 PuppeteerSuperPrivs,
		 PuppeteerModPrivs)
		VALUES (?,?,?,?,?,?,?,?,'no','no','no')", {}, 

  	  $param->{'USERNAME'}, 
	  crypt(substr($param->{'PASSONCE'}, 0, 8), $confcode), 
	  $param->{'FULLNAME'}, 
          $param->{'EMAIL'}, 
          time,
	  $option->RegFormatting($login{'OptFormat'},$param->{'FORMATPREFS'}),
	  $option->RegChatPrefs($login{'OptChatPref'},$param->{'CHATPREFS'}),
	  $option->RegJavascript($login{'OptJavascript'},$param->{'JAVASCRIPT'}));

	# Registration codes
	$data->do("INSERT INTO PuppeteerRegistration 
		(PuppeteerLogin,
		 PuppeteerConfCode,
		 PuppeteerTimeStamp)
		VALUES (?,?,?)", {}, 
	 $param->{'USERNAME'}, $confcode, time);


	# Puppets
	# Create Puppet
	$data->do("INSERT INTO Puppet
		(PuppetName,
		 PuppeteerLogin)
		VALUES (?,?)", {},
	 "Puppet ($confcode)",
	 $param->{'USERNAME'});
}

